/*
 * OpenGL(R) is a registered trademark of Silicon Graphics, Inc.
 */

#include "main.h"
#include "tristrip.h"
#include "tunnelqueue.h"
#include "tunnel.h"
#include "ship.h"
#include "badguycontroller.h"
#include "shotcontroller.h"

float rotx, roty, rotz;
float rotzoffset;
int mouseLastx, mouseDown;
float oldrotz = 0;
TunnelQueue * tunnelQueue;
Tunnel * tunnel;
Ship * ship;
ShotController * shotcontrol;
BadGuyController * badguycontrol;
float ztemp = 0;
int frame;
GLubyte Texture1[128*128*3];
GLubyte Texture2[128*128*3];
GLubyte Texture3[16][16][3];
int do_textures = 1;

static char *circles[] = {
  "....xxxx........",
  "..xxxxxxxx......",
  ".xxxxxxxxxx.....",
  ".xxx....xxx.....",
  "xxx......xxx....",
  "xxx......xxx....",
  "xxx......xxx....",
  "xxx......xxx....",
  ".xxx....xxx.....",
  ".xxxxxxxxxx.....",
  "..xxxxxxxx......",
  "....xxxx........",
  "................",
  "................",
  "................",
  "................",
};

 
void init(void) 
	{
	glPushMatrix();
	static float fog_color[] = {0.0, 0.0, 0.0, 1.0};
	int i;
	FILE *file;

   	GLfloat position0[] = {0, 0, -16, 1.0};
	GLfloat position1[] = { 0, -1.0, -6, 1.0};
	GLfloat position2[] = { 0, -0.5, 1, 1.0};

	GLfloat light0dif[] = { .5, .5, .5, 1.0 };
	GLfloat light1dif[] = { .4, .4, .4, 1.0 };
	GLfloat light2dif[] = { .5, .5, .5, 1.0 };

	GLfloat light0spec[] = { .3, .3, .3, 1.0 };
	GLfloat light1spec[] = { .1, .1, .1, 1.0 };
	GLfloat light2spec[] = { .1, .1, .1, 1.0 };

   	//GLfloat amb[] = {1, 1, 1, 1.0};
   	//GLfloat amb[] = {0.1, 0.1, 0.1, 1.0};
   	//GLfloat amb[] = {0.0, 0.0, 0.0, 1.0};
   	//GLfloat lmamb[] = {0.2,0.2,0.2, 1.0};
   	GLfloat lmamb[] = {0.1,0.1,0.1, 1.0};
   	//GLfloat spec[] = {.01,.01,.01,1};
   	//GLfloat shin = {1.0};
   	glClearColor (0.0, 0.0, 0.0, 0.0);

   	//glMaterialfv(GL_FRONT, GL_AMBIENT, amb);
   	//glMaterialfv(GL_FRONT, GL_SPECULAR, spec);
   	//glMaterialf (GL_FRONT,GL_SHININESS, shin);
	
   	glLightfv (GL_LIGHT0, GL_POSITION, position0);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, light0dif);
	glLightfv (GL_LIGHT0, GL_SPECULAR, light0spec);
	
	glLightfv (GL_LIGHT1, GL_POSITION, position1);
	glLightfv (GL_LIGHT1, GL_DIFFUSE, light1dif);
	glLightfv (GL_LIGHT1, GL_SPECULAR, light1spec);

	glLightfv (GL_LIGHT2, GL_POSITION, position2);
	glLightfv (GL_LIGHT2, GL_DIFFUSE, light2dif);
	glLightfv (GL_LIGHT2, GL_SPECULAR, light2spec);
   	
	glLightModelfv (GL_LIGHT_MODEL_AMBIENT, lmamb);
   	//glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
	
   	glEnable(GL_LIGHTING);
   	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);
	glEnable(GL_LIGHT2);
   	glEnable(GL_DEPTH_TEST);
   	glShadeModel ( SHADE_MODE );
   	//glColorMaterial(GL_FRONT, GL_DIFFUSE);
   	glEnable(GL_COLOR_MATERIAL);

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

  file = fopen("texture1.rgb", "rb");
  for (i = 0; i < 128; i++) {
    fread(Texture1, 1, 128*128*3, file);
  }
  fclose(file);

  file = fopen("texture2.rgb", "rb");
  for (i = 0; i < 128; i++) {
    fread(Texture2, 1, 128*128*3, file);
  }
  fclose(file);

  GLubyte *loc;
  int s, t;

  /* Setup RGB image for the texture. */
  loc = (GLubyte*) Texture3;
  for (t = 0; t < 16; t++) {
    for (s = 0; s < 16; s++) {
      if (circles[t][s] == 'x') {
        /* Nice blue. */
        loc[0] = 0x1f;
        loc[1] = 0x1f;
        loc[2] = 0x8f;
      } else {
        /* Light gray. */
        loc[0] = 0xca;
        loc[1] = 0xca;
        loc[2] = 0xca;
      }
      loc += 3;
    }
  }

	glEnable(GL_FOG);
	glFogi(GL_FOG_MODE, GL_EXP);
	glFogf(GL_FOG_DENSITY, FOG_DENSITY);
	glFogfv(GL_FOG_COLOR, fog_color);
	glPopMatrix();
	}

void display(void)
	{
	frame++;

	//GLfloat diff[] = {.7,.7,.7, 1.0};

   	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   	// Do rest of world
	glPushMatrix();
	//Do world rotations
	glTranslatef( 0, 1.0, 0 );
	rotz += rotzoffset;
	glRotatef ( rotz, 0, 0, 1 );
	
	// Update any shots we have fired
	shotcontrol->update();
	
	// Update the tunnel (add/delete sections to queue)
	tunnel->update();
	// Update bad guys (queue new bad guys, dequeue dead bad guys
	//  and bad guys that are passed the player
	badguycontrol->update();
	
	// Go through display queue
	tunnelQueue->drawAll();
	glPopMatrix();

	// Do main character
	glPushMatrix();
	glTranslatef( 0, -1.1, -1.5 );
	glScalef(0.005, 0.005, 0.005);
	glRotatef( 180, 0, 1, 0);

	if ( rotz - oldrotz > 0 ) glRotatef( -5, 0, 0, 1);
	else if ( rotz - oldrotz < 0 ) glRotatef( 5, 0, 0, 1);
	oldrotz = rotz;

	ship->draw();
	glPopMatrix();

   	//getchar();
   	//printf("swap buffers\n");
	glutSwapBuffers();
	}

void reshape (int w, int h)
	{
   	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
   	glMatrixMode (GL_PROJECTION);
   	glLoadIdentity ();
   	//gluPerspective(65.0, (GLfloat) w/(GLfloat) h, 1.0, 20.0);
   	gluPerspective(100.0, (GLfloat) w/(GLfloat) h, 1.0, 20.0);
   	glMatrixMode(GL_MODELVIEW);
   	glLoadIdentity();
   	//glTranslatef (0.0, 0.0, -5.0);
	}

void keyboard (unsigned char key, int x, int y)
	{
	switch(key)
		{
		case QUIT: 
			exit(42);
			break;
		case GOLEFT:
			//rotzoffset += TURN_RATE;
			rotzoffset = TURN_RATE;
			break;
		case GORIGHT:
			//rotzoffset -= TURN_RATE;
			rotzoffset = -1*TURN_RATE;
			break;
		case RECENTER:
			rotzoffset = 0;
			break;
		case FIRE:
			shotcontrol->fire();
			break;
		case 't':
			do_textures = !do_textures;
			break;
		default:
			break;
		}	
	}

void keyboardSpecial (int key, int x, int y)
	{
	switch(key)
		{
		case GOLEFT_SPECIAL:
			//rotzoffset += TURN_RATE;
			rotzoffset = TURN_RATE;
			break;
		case GORIGHT_SPECIAL:
			//rotzoffset -= TURN_RATE;
			rotzoffset = -1*TURN_RATE;
			break;
		case RECENTER_SPECIAL:
			rotzoffset = 0;
			break;
		default:
			break;
		}
	}	

void motion( int x, int y )
	{
	rotz -= x - mouseLastx;
	mouseLastx = x;
	}

void mouse( int button, int state, int x, int y)
	{
	mouseLastx = x;
	if (button == GLUT_LEFT_BUTTON) 
		{
		if (state == GLUT_DOWN) 
			{
			shotcontrol->fire();
			mouseDown = 1;
			} 
		else 
			{
			mouseDown = 0;
			}
		}	
	}

void idle()
	{
	// should only call this if enough time has passed
	display();
	}

int main(int argc, char** argv)
	{
	frame = 0;
	tunnelQueue = new TunnelQueue();
	tunnel = new Tunnel( tunnelQueue );
	ship = new Ship();
	shotcontrol = new ShotController( tunnelQueue );
	badguycontrol = new BadGuyController( tunnelQueue, shotcontrol );
	
	rotx = roty = rotz = rotzoffset = 0;

   	glutInit(&argc, argv);
   	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   	glutInitWindowSize (WINDOW_X, WINDOW_Y); 
   	glutInitWindowPosition (0, 0);
   	glutCreateWindow (argv[0]);
   	init();
   	glutDisplayFunc(display); 
   	glutReshapeFunc(reshape);
   	glutIdleFunc(idle);
	// do keyboard
   	glutKeyboardFunc(keyboard);
	glutSpecialFunc(keyboardSpecial);
	// do mouse
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	glutPassiveMotionFunc(motion);
	// go to town
   	glutMainLoop();
   	return 0;
	}
