#include "tunnel.h"

void Tunnel::update()
	{
	int id;
	int stripId;
	addDelay++;
	if ( addDelay > (ADD_SIDE_DELAY) )
		{
		timer++;
		if ( timer > TIMER_VALUE )
			{
			//dequeue old
			//printf("dequeueing %d at %d\n", 
			//	stripIds[(stripIdPtr + 1) % TUNNEL_LENGTH ],
			//	(stripIdPtr + 1) % TUNNEL_LENGTH);

			stripId = stripIds[ (stripIdPtr + 1) % TUNNEL_LENGTH ];
			//printf("dequeueing tunnelstrip %d\n", stripId);
			tunnelQueue->deQueue( stripId );

			TriStrip * tris = new TriStrip();
			genSides();
			// Do colors
			genColours();
			tris->setColours( colours );
			tris->setStripCoords( stripCoords );
			tris->setFrontCoordAvg( frontCoordAvg );
			tris->setBackCoordAvg( backCoordAvg );
			id = tunnelQueue->queue( tris );
			stripIds[ stripIdPtr ] = id;
			//printf("queueing strip %d at %d\n", id, stripIdPtr);
			tris->setIdnum( id );
			//printf( "Making new strip, id: %d strip: %d contents\n", id, stripCoords );
			addDelay = 0;
			stripIdPtr = (stripIdPtr + 1) % TUNNEL_LENGTH;
			}
		}
	}

void Tunnel::genSides()
	{
	newPoints = lastPoints;
	pointRandomize();
	genTriStripCoords();
	lastPoints = newPoints;
	}

// Make the initial standard side points that the rest will be based off
void Tunnel::genFirstSides()
	{
	float degreeOffset = (2 * 3.1415) / TUNNEL_SIDES;
	float theta;
	for ( int i = 0; i < TUNNEL_SIDES + 1; i++ )
		{
		theta = degreeOffset * i;
		// Take care of last point, which is the same as the first
		if ( i == TUNNEL_SIDES ) 
			{
			theta = 0;
			}
		lastPoints[ 0 ][ i ] = (float)(cos( theta ) * TUNNEL_RADIUS);
		lastPoints[ 1 ][ i ] = (float)(sin( theta ) * TUNNEL_RADIUS);
		lastPoints[ 2 ][ i ] = 8;

		initPoints[ 0 ][ i ] = lastPoints[ 0 ][ i ];
		initPoints[ 1 ][ i ] = lastPoints[ 1 ][ i ];
		initPoints[ 2 ][ i ] = lastPoints[ 2 ][ i ];
		//printf( "xyz: %f %f %f\ttheta: %f degreeoffset: %f\n", 
		//		lastPoints[0][i], lastPoints[1][i], lastPoints[2][i],
		//		theta, degreeOffset );
		}
	}
	
void Tunnel::genTriStripCoords()
	{
	int place;
	stripCoords = new float [ 3 * 3 * (TUNNEL_SIDES + 1) ];
	for ( int i = 0; i < 2 * (TUNNEL_SIDES + 1); i++ )
		{
		if ( (i%2) == 0 )
			{
			place = i / 2;
			stripCoords[ i ] = lastPoints[ 0 ][ place ];
			stripCoords[ i + 2*(TUNNEL_SIDES + 1) ] = lastPoints[ 1 ][ place ];
			stripCoords[ i + 4*(TUNNEL_SIDES + 1) ] = lastPoints[ 2 ][ place ];
			}
		else
			{
			place = (i - 1) / 2;
			stripCoords[ i ] = newPoints[ 0 ][ place ];
			stripCoords[ i + 2*(TUNNEL_SIDES + 1) ] = newPoints[ 1 ][ place ];
			stripCoords[ i + 4*(TUNNEL_SIDES + 1) ] = newPoints[ 2 ][ place ];
			}
		
		//printf( "Tunnel::genTriStrip stripCoords: %f %f %f\n", 
		//		stripCoords[ i ], stripCoords[ 1 * i ], stripCoords[ 2 * i ] );
		}
	}
	
	void Tunnel::genColours()
		{
		int id = tunnelQueue->getIdnum();
		colours = new float [ 3 * 2 * (TUNNEL_SIDES + 1) ];
		float r = 0;
		float g = 0;
		float b = 0;
		if ( id % 3 == 0 )
			{
			r = .5;
			g = .3;
			b = .3;
			}
		else if ( id % 3 == 1 )
			{
			r = .7;
			g = .5;
			b = .5;
			}
		else
			{
			r = .6;
			g = .4;
			b = .4;
			}
		float rn, gn, bn;	
		for ( int i = 0; i < 2*(TUNNEL_SIDES + 1); i++ )
			{
			//float fl = (float)i;
			// this did the fading as it went around the z axis
			//if ( fl > TUNNEL_SIDES + 1 )
			//	{
			//	fl = 2*(TUNNEL_SIDES + 1) - fl;
			//	}
			//rn = r * ( fl / (TUNNEL_SIDES + 1) );
			//gn = g * ( fl / (TUNNEL_SIDES + 1) );
			//bn = b * ( fl / (TUNNEL_SIDES + 1) );
			
			rn = ( (float)(rand()/(RAND_MAX+1.0)) );
			gn = bn = rn + .2;
			if ( gn > 1 ) gn = bn = 1;
			//rn = gn = bn = .5;
			//gn = ( (float)(rand()/(RAND_MAX+1.0)) );
			//bn = ( (float)(rand()/(RAND_MAX+1.0)) );
			
			//printf( "colours: rgb: %f %f %f, %f\n", rn, gn, bn, fl);
			colours[ i ] = rn;
			colours[ i + 2*(TUNNEL_SIDES + 1) ] = gn;
			colours[ i + 4*(TUNNEL_SIDES + 1) ] = bn;
			}

		}

	void Tunnel::pointRandomize()
		{
		//printf("deleting frontCoordAvg\n");
		if(frontCoordAvg != NULL) delete frontCoordAvg;
		frontCoordAvg = backCoordAvg;
		backCoordAvg = new float [3];
		for ( int i = 0; i < TUNNEL_SIDES; i++ )
			{
			//printf( "rand %f\n", (float)(rand()/(RAND_MAX+1.0)) );
			float xmod = ((float)(rand() / (RAND_MAX+1.0)) - .5)*1;
			float ymod = ((float)(rand() / (RAND_MAX+1.0)) - .5)*1; 
			newPoints[0][i] = initPoints[0][i] + xmod;
			newPoints[1][i] = initPoints[1][i] + ymod;
			newPoints[2][i] -= STRIP_DEPTH;
			//printf("lines %f %f %f\n", newPoints[0][i], newPoints[1][i], newPoints[2][i]);
			backCoordAvg[0] += newPoints[0][i];
			backCoordAvg[1] += newPoints[1][i];
			backCoordAvg[2] += newPoints[2][i];
			//printf( " z: %f total: %f\n", newPoints[2][i], backCoordAvg[2]);
			}
		// Get rid of tear	
		newPoints[0][TUNNEL_SIDES] = newPoints[0][0];
		newPoints[1][TUNNEL_SIDES] = newPoints[1][0];
		newPoints[2][TUNNEL_SIDES] = newPoints[2][0];

		// Devide averages
		backCoordAvg[0] /= TUNNEL_SIDES;
		backCoordAvg[1] /= TUNNEL_SIDES;
		backCoordAvg[2] /= TUNNEL_SIDES;
		}
